package edu.illinois.cs.cogcomp.cs546ccm2.evaluation.BAT.Metrics;

/**
 * Adopted by Shashank from BAT-Framework
 */

import java.util.List;

import edu.illinois.cs.cogcomp.cs546ccm2.evaluation.BAT.MatchCriteria.RelatednessMeasurer;

public class RelatednessMetrics {

	private static float[] getOutput(List<RelatednessMetricsRecord> output, RelatednessMeasurer rel) {
		float[] outputRel = new float[output.size()];
		for (int i = 0; i < output.size(); i++) {
			RelatednessMetricsRecord rec = output.get(i);
			outputRel[i] = (rel.getRelatedness(rec.getEntity1(), rec.getEntity2()));
			if (outputRel[i] > 1 || outputRel[i] < 0) {
				throw new RuntimeException("Both relatedness measures should be in [0,1]. output=" + outputRel[i]);
			}
		}
		return outputRel;
	}

	private static float[] getGoldStandard(List<RelatednessMetricsRecord> goldStandard) {
		float[] goldStandardRel = new float[goldStandard.size()];
		for (int i = 0; i < goldStandard.size(); i++) {
			RelatednessMetricsRecord rec = goldStandard.get(i);
			goldStandardRel[i] = rec.getRelatedness();
			if (goldStandardRel[i] > 1 || goldStandardRel[i] < 0) {
				throw new RuntimeException("Both relatedness measures should be in [0,1]. goldStandard=" + goldStandardRel[i]);
			}
		}
		return goldStandardRel;
	}

	public static float getQuadraticDistance(List<RelatednessMetricsRecord> goldStandardList, RelatednessMeasurer rel) {
		float[] output = getOutput(goldStandardList, rel);
		float[] goldStandard = getGoldStandard(goldStandardList);
		if (output.length != goldStandard.length) {
			throw new NullPointerException();
		}
		float res = 0;
		for (int i = 0; i < output.length; i++) {
			res += Math.pow((output[i] - goldStandard[i]), 2);
		}
		res /= output.length;
		return 1 - (float) Math.sqrt(res);
	}

	public static float getAbsoluteDistance(List<RelatednessMetricsRecord> goldStandardList, RelatednessMeasurer rel) {
		float[] output = getOutput(goldStandardList, rel);
		float[] goldStandard = getGoldStandard(goldStandardList);
		if (output.length != goldStandard.length) {
			throw new NullPointerException();
		}
		float res = 0;
		for (int i = 0; i < output.length; i++) {
			res += Math.abs(output[i] - goldStandard[i]);
		}
		res /= output.length;
		return 1 - res;
	}

}
